@@ -7,6 +7,8 @@ import pytest
 from werkzeug import routing as r
 from werkzeug.datastructures import ImmutableDict
 from werkzeug.datastructures import MultiDict
+from werkzeug.exceptions import MethodNotAllowed
+from werkzeug.exceptions import NotFound
 from werkzeug.test import create_environ
 from werkzeug.wrappers import Response
 
@@ -26,7 +28,7 @@ def test_basic_routing():
     assert adapter.match("/foo") == ("foo", {})
     assert adapter.match("/bar/") == ("bar", {})
     pytest.raises(r.RequestRedirect, lambda: adapter.match("/bar"))
-    pytest.raises(r.NotFound, lambda: adapter.match("/blub"))
+    pytest.raises(NotFound, lambda: adapter.match("/blub"))
 
     adapter = map.bind("example.org", "/", url_scheme="ws")
     assert adapter.match("/") == ("indexws", {})
@@ -171,7 +173,7 @@ def test_strict_slashes_redirect():
 
     # Check if exceptions are correct
     pytest.raises(r.RequestRedirect, adapter.match, "/bar", method="GET")
-    pytest.raises(r.MethodNotAllowed, adapter.match, "/bar/", method="POST")
+    pytest.raises(MethodNotAllowed, adapter.match, "/bar/", method="POST")
     with pytest.raises(r.RequestRedirect) as error_info:
         adapter.match("/foo", method="POST")
     assert error_info.value.code == 308
@@ -191,7 +193,7 @@ def test_strict_slashes_redirect():
 
     # Check if exceptions are correct
     pytest.raises(r.RequestRedirect, adapter.match, "/bar", method="GET")
-    pytest.raises(r.MethodNotAllowed, adapter.match, "/bar/", method="POST")
+    pytest.raises(MethodNotAllowed, adapter.match, "/bar/", method="POST")
 
     # Check what happens when only slash route is defined
     map = r.Map([r.Rule("/bar/", endpoint="get", methods=["GET"])])
@@ -202,8 +204,37 @@ def test_strict_slashes_redirect():
 
     # Check if exceptions are correct
     pytest.raises(r.RequestRedirect, adapter.match, "/bar", method="GET")
-    pytest.raises(r.MethodNotAllowed, adapter.match, "/bar/", method="POST")
-    pytest.raises(r.MethodNotAllowed, adapter.match, "/bar", method="POST")
+    pytest.raises(MethodNotAllowed, adapter.match, "/bar/", method="POST")
+
+
+def test_strict_slashes_leaves_dont_consume():
+    # See issue #1074
+    map = r.Map(
+        [
+            r.Rule("/path1", endpoint="leaf"),
+            r.Rule("/path1/", endpoint="branch"),
+            r.Rule("/path2", endpoint="leaf", strict_slashes=False),
+            r.Rule("/path2/", endpoint="branch"),
+            r.Rule("/path3", endpoint="leaf"),
+            r.Rule("/path3/", endpoint="branch", strict_slashes=False),
+            r.Rule("/path4", endpoint="leaf", strict_slashes=False),
+            r.Rule("/path4/", endpoint="branch", strict_slashes=False),
+            r.Rule("/path5", endpoint="leaf"),
+        ],
+        strict_slashes=False,
+    )
+
+    adapter = map.bind("example.org", "/")
+
+    assert adapter.match("/path1", method="GET") == ("leaf", {})
+    assert adapter.match("/path1/", method="GET") == ("branch", {})
+    assert adapter.match("/path2", method="GET") == ("leaf", {})
+    assert adapter.match("/path2/", method="GET") == ("branch", {})
+    assert adapter.match("/path3", method="GET") == ("leaf", {})
+    assert adapter.match("/path3/", method="GET") == ("branch", {})
+    assert adapter.match("/path4", method="GET") == ("leaf", {})
+    assert adapter.match("/path4/", method="GET") == ("branch", {})
+    assert adapter.match("/path5/", method="GET") == ("leaf", {})
 
 
 def test_environ_defaults():
@@ -214,7 +245,7 @@ def test_environ_defaults():
     assert a.match("/foo") == ("foo", {})
     assert a.match() == ("foo", {})
     assert a.match("/bar") == ("bar", {})
-    pytest.raises(r.NotFound, a.match, "/bars")
+    pytest.raises(NotFound, a.match, "/bars")
 
 
 def test_environ_nonascii_pathinfo():
@@ -223,7 +254,7 @@ def test_environ_nonascii_pathinfo():
     a = m.bind_to_environ(environ)
     assert a.match("/") == ("index", {})
     assert a.match("/лошадь") == ("horse", {})
-    pytest.raises(r.NotFound, a.match, "/барсук")
+    pytest.raises(NotFound, a.match, "/барсук")
 
 
 def test_basic_building():
@@ -330,10 +361,10 @@ def test_negative():
     assert adapter.match("/bars/-0.185") == ("bars", {"page": -0.185})
 
     # Make sure signed values are rejected in unsigned mode
-    pytest.raises(r.NotFound, lambda: adapter.match("/foo/-2"))
-    pytest.raises(r.NotFound, lambda: adapter.match("/foo/-50"))
-    pytest.raises(r.NotFound, lambda: adapter.match("/bar/-0.185"))
-    pytest.raises(r.NotFound, lambda: adapter.match("/bar/-2.0"))
+    pytest.raises(NotFound, lambda: adapter.match("/foo/-2"))
+    pytest.raises(NotFound, lambda: adapter.match("/foo/-50"))
+    pytest.raises(NotFound, lambda: adapter.match("/bar/-0.185"))
+    pytest.raises(NotFound, lambda: adapter.match("/bar/-2.0"))
 
 
 def test_greedy():
@@ -435,8 +466,8 @@ def test_dispatch():
 
     assert dispatch("/").data == b"('root', {})"
     assert dispatch("/foo").status_code == 308
-    raise_this = r.NotFound()
-    pytest.raises(r.NotFound, lambda: dispatch("/bar"))
+    raise_this = NotFound()
+    pytest.raises(NotFound, lambda: dispatch("/bar"))
     assert dispatch("/bar", True).status_code == 404
 
 
@@ -720,6 +751,8 @@ def test_converter_with_tuples():
     """
 
     class TwoValueConverter(r.BaseConverter):
+        part_isolating = False
+
         def __init__(self, *args, **kwargs):
             super().__init__(*args, **kwargs)
             self.regex = r"(\w\w+)/(\w\w+)"
@@ -754,6 +787,19 @@ def test_anyconverter():
     assert a.match("/a.2") == ("yes_dot", {"a": "a.2"})
 
 
+def test_any_converter_build_validates_value() -> None:
+    m = r.Map([r.Rule("/<any(patient, provider):value>", endpoint="actor")])
+    a = m.bind("localhost")
+
+    assert a.build("actor", {"value": "patient"}) == "/patient"
+    assert a.build("actor", {"value": "provider"}) == "/provider"
+
+    with pytest.raises(ValueError) as exc:
+        a.build("actor", {"value": "invalid"})
+
+    assert str(exc.value) == "'invalid' is not one of 'patient', 'provider'"
+
+
 @pytest.mark.parametrize(
     ("endpoint", "value", "expect"),
     [
@@ -901,7 +947,7 @@ def test_implicit_head():
     )
     adapter = url_map.bind("example.org")
     assert adapter.match("/get", method="HEAD") == ("a", {})
-    pytest.raises(r.MethodNotAllowed, adapter.match, "/post", method="HEAD")
+    pytest.raises(MethodNotAllowed, adapter.match, "/post", method="HEAD")
 
 
 def test_pass_str_as_router_methods():
@@ -1101,7 +1147,7 @@ def test_server_name_casing():
     with pytest.warns(UserWarning):
         a = m.bind_to_environ(env, server_name="example.com")
 
-    with pytest.raises(r.NotFound):
+    with pytest.raises(NotFound):
         a.match()
 
 
@@ -1181,7 +1227,7 @@ def test_both_bind_and_match_path_info_are_none():
 def test_map_repr():
     m = r.Map([r.Rule("/wat", endpoint="enter"), r.Rule("/woop", endpoint="foobar")])
     rv = repr(m)
-    assert rv == "Map([<Rule '/woop' -> foobar>, <Rule '/wat' -> enter>])"
+    assert rv == "Map([<Rule '/wat' -> enter>, <Rule '/woop' -> foobar>])"
 
 
 def test_empty_subclass_rules_with_custom_kwargs():
@@ -1348,9 +1394,36 @@ def test_rule_websocket_methods():
     r.Rule("/ws", endpoint="ws", websocket=True, methods=["get", "head", "options"])
 
 
+def test_path_weighting():
+    m = r.Map(
+        [
+            r.Rule("/<path:path>/c", endpoint="simple"),
+            r.Rule("/<path:path>/<a>/<b>", endpoint="complex"),
+        ]
+    )
+    a = m.bind("localhost", path_info="/a/b/c")
+
+    assert a.match() == ("simple", {"path": "a/b"})
+
+
 def test_newline_match():
     m = r.Map([r.Rule("/hello", endpoint="hello")])
     a = m.bind("localhost")
 
-    with pytest.raises(r.NotFound):
+    with pytest.raises(NotFound):
         a.match("/hello\n")
+
+
+def test_weighting():
+    m = r.Map(
+        [
+            r.Rule("/<int:value>", endpoint="int"),
+            r.Rule("/<uuid:value>", endpoint="uuid"),
+        ]
+    )
+    a = m.bind("localhost")
+
+    assert a.match("/2b5b0911-fdcf-4dd2-921b-28ace88db8a0") == (
+        "uuid",
+        {"value": uuid.UUID("2b5b0911-fdcf-4dd2-921b-28ace88db8a0")},
+    )
